import { memo } from 'react';

import Box from '@mui/material/Box';
import { NsTypography, Show, Variants } from '@nuskin/foundation-ui-components';

import { BynderImage } from '../../../bynder-image/BynderImage';
import NsCTALinkComponent from '../../../cta-link/CtaLink';
import useToggleFontColor from '../../../hooks/use-toggle-font-color/useToggleFontColor';
import { getBasicDATWidthHeight } from '../../../utils/bynder-util';
import { HeroCardProps } from '../../types/types';
import { ContentContainer, ImageContainer, StyledLink, TextContainer } from './ThreeCards.styled';

const PanelTypographyVariantMap: {
    [key: string]: {
        preHeaderVariant: Variants;
        headerVariant: Variants;
        descriptionVariant: Variants;
    };
} = {
    main: {
        preHeaderVariant: 'h4',
        headerVariant: 'h1',
        descriptionVariant: 'h3',
    },
    sub: {
        preHeaderVariant: 'body-m',
        headerVariant: 'h2',
        descriptionVariant: 'body-l',
    },
};

/**
 * Hero card specifically for Three cards component. This is a memod component.
 * TODO: we can move this out if it can be reused. Rename it appropriately.
 */
const suffix = getBasicDATWidthHeight(900, 480, 'scale');
const HeroCard = memo((props: HeroCardProps & { isMain?: boolean }) => {
    const {
        preHeader,
        header,
        description,
        fontColor,
        backgroundColor,
        backgroundScrim,
        image,
        callToAction,
        renderH1 = false,
        liveTags,
        dataTestId = 'hero-card',
        isMain,
        $,
    } = props;

    const { url } = callToAction || {};

    const { preHeaderVariant, headerVariant, descriptionVariant } = PanelTypographyVariantMap[isMain ? 'main' : 'sub'];

    const resolvedFontColor = useToggleFontColor(fontColor);

    const imageSection = (
        <ImageContainer>
            <BynderImage
                liveTag={$?.image ?? { 'data-cslp': '' }}
                imageText={image}
                suffix={suffix}
                className="three-card-hero-image"
            />
        </ImageContainer>
    );

    const textSection = (
        <TextContainer>
            <Box sx={{ overflow: 'hidden', height: '100%' }}>
                <Show when={!!preHeader}>
                    <NsTypography
                        variant={preHeaderVariant}
                        colorOverride={resolvedFontColor}
                        additionalProps={{
                            'data-testid': 'pre-header',
                            ...liveTags?.preHeader,
                        }}
                        noSpacing
                    >
                        {preHeader}
                    </NsTypography>
                </Show>
                <Show when={!!header}>
                    <NsTypography
                        variant={headerVariant}
                        renderHTMLNode={renderH1}
                        colorOverride={resolvedFontColor}
                        additionalProps={{
                            'data-testid': 'header',
                            ...liveTags?.header,
                        }}
                    >
                        {header}
                    </NsTypography>
                </Show>
                <Show when={!!description}>
                    <NsTypography
                        variant={descriptionVariant}
                        colorOverride={resolvedFontColor}
                        additionalProps={{
                            'data-testid': 'description',
                            ...liveTags?.description,
                        }}
                    >
                        {description}
                    </NsTypography>
                </Show>
                <Show when={!!callToAction?.text}>
                    <div {...liveTags?.callToAction}>
                        <NsCTALinkComponent ctaProps={callToAction} />
                    </div>
                </Show>
            </Box>
        </TextContainer>
    );

    const className = ['hero-card', isMain ? 'main-card' : 'sub-card', backgroundScrim ? 'with-scrim' : false]
        .filter(Boolean)
        .join(' ');
    const content = (
        <ContentContainer backgroundColor={backgroundColor}>
            {imageSection}
            {textSection}
        </ContentContainer>
    );

    return (
        <div className={className} data-testid={dataTestId}>
            {url ? (
                <StyledLink href={url} target="_blank" data-testid="link">
                    {content}
                </StyledLink>
            ) : (
                content
            )}
        </div>
    );
});

HeroCard.displayName = 'HeroCard';

export default HeroCard;
