/* eslint-disable no-case-declarations */
import { OptionsProps } from '@nuskin/foundation-ui-components';

export interface SearchReducerDataType {
    loading: boolean;
    searchTerm: string;
    selectedOption: string | OptionsProps | null;
    hasMorePages: boolean;
    acSearchOptions: OptionsProps[];
    didYouMean: OptionsProps[];
    pageInfo: {
        size: number; // set size of each response
        count: number; // result count
        totalCount: number; // total entries in db
        page: number; // current page number
    };
}
export type Action = {
    type: string;
    payload?: any;
};

export const ActionTypes = {
    FETCH_START: 'FETCH_START',
    FETCH_SUCCESS: 'FETCH_SUCCESS',
    SEARCH_TERMS: 'SEARCH_TERMS',
    SELECTED_OPTION: 'SELECTED_OPTION',
    RESET: 'RESET',
    SET_AC_SEARCH_OPTIONS: 'SET_AC_SEARCH_OPTIONS',
    INCREMENT_PAGE: 'INCREMENT_PAGE'
};
const initialPageInfo: SearchReducerDataType['pageInfo'] = {
    size: 1,
    count: 0,
    totalCount: 0,
    page: 1
};
export const initialSearchAutoCompleteState: SearchReducerDataType = {
    loading: false,
    searchTerm: '',
    selectedOption: null,
    pageInfo: initialPageInfo,
    hasMorePages: false,
    acSearchOptions: [],
    didYouMean: []
};
export const searchGroups = {
    didYouMean: 'Did You Mean ?',
    noResults: 'No Result Found'
};
const computeAcSearchOptions = (
    state: SearchReducerDataType,
    payload: {
        pageInfo: SearchReducerDataType['pageInfo'];
        didYouMean: OptionsProps[];
        errorMessage?: string;
        productList: OptionsProps[];
    }
) => {
    const { pageInfo, productList = [], errorMessage } = payload;
    if (errorMessage) {
        return [
            {
                id: searchGroups.noResults,
                value: '',
                group: `No result found for ${state.searchTerm}`
            }
        ];
    } else if (state.searchTerm === '') {
        // CX130-881 sometimes data is returned late, but till then search value is already empty
        return [];
    } else {
        let options = productList;
        if (pageInfo?.page > 1) {
            options = [...state.acSearchOptions, ...productList];
        } else {
            options = productList;
        }
        const { totalCount, size, page } = pageInfo;
        const hasMorePages = totalCount > page * size;

        return options.map((option) => ({
            ...option,
            group: `${options.length}${hasMorePages ? '+' : ''} search suggestions for ${state.searchTerm}`
        }));
    }
};
export const searchAutoCompleteReducer = (state: SearchReducerDataType, action: Action) => {
    switch (action.type) {
        case ActionTypes.FETCH_START:
            return {
                ...state,
                loading: true
            };
        case ActionTypes.FETCH_SUCCESS:
            const pageInfo = action.payload?.pageInfo || initialPageInfo;
            const { totalCount, size, page } = pageInfo;
            const hasMorePages = totalCount > page * size;

            return {
                ...state,
                loading: false,
                hasMorePages,
                pageInfo,
                didYouMean: action.payload?.didYouMean || [],
                acSearchOptions: computeAcSearchOptions(state, action.payload)
            };
        case ActionTypes.SEARCH_TERMS:
            return {
                ...state,
                searchTerm: action.payload,
                pageInfo: { ...state.pageInfo, page: 1 },
                acSearchOptions: action.payload === '' ? [] : state.acSearchOptions
            };
        case ActionTypes.SELECTED_OPTION:
            return {
                ...state,
                selectedOption: action.payload
            };
        case ActionTypes.RESET:
            return initialSearchAutoCompleteState;
        case ActionTypes.INCREMENT_PAGE:
            return {
                ...state,
                pageInfo: {
                    ...state.pageInfo,
                    page: state.pageInfo.page + 1
                }
            };
        case ActionTypes.SET_AC_SEARCH_OPTIONS:
            return {
                ...state,
                acSearchOptions: state.searchTerm === '' ? [] : action.payload
            };
        default:
            throw new Error(`Unsupported action type: ${action.type}`);
    }
};
