import {useEffect, useState} from 'react';

type ValueTypes = string | any[];

/**
 *
 * @param key  key as a string to identify the item in localStorage,
 * @param initialValue initialValue to use when no value is present in localStorage for the given key.
 * @returns The hook returns an array with two elements: the current value from localStorage
 *  and a function setValue to update the value. You can use this function to update the value as needed.
 */

const isLocalStorageAvailable = (): boolean => {
    try {
        const testKey = '__test_key__';
        localStorage.setItem(testKey, testKey);
        localStorage.removeItem(testKey);
        return true;
    } catch (error) {
        return false;
    }
};

const useLocalStorage = <T extends ValueTypes>(
    key: string,
    initialValue: T
): [T, (value: T) => void] => {
    const [value, setValue] = useState<T>(() => {
        if (isLocalStorageAvailable()) {
            const storedValue = localStorage.getItem(key);

            if (storedValue !== null && storedValue !== 'undefined') {
                try {
                    return JSON.parse(storedValue) as T;
                } catch (error) {
                    // Handle parsing error if necessary
                    console.error('Error parsing stored value:', error);
                }
            }
        }

        return initialValue;
    });

    useEffect(() => {
        try {
            const serializedValue = JSON.stringify(value);
            localStorage.setItem(key, serializedValue);
        } catch (error) {
            // Handle serialization error if necessary
            console.error('Error serializing value:', error);
        }
    }, [key, value]);

    return [value, setValue];
};

export default useLocalStorage;
